<?php

/**
 * WAAVE Complete Gateway
 *
 * We load it later to ensure WC is loaded first since we're extending it.
 *
 * @class       WC_WAAVE_Complete
 * @extends     WC_Payment_Gateway
 * @version     2.0.0
 * @package     WooCommerce/Classes/Payment
 */
class WC_WAAVE_Checkout extends WC_Payment_Gateway {

    const PROD_URL = 'https://pg.getwaave.co';
    const SANDBOX_URL = 'https://staging-pg.getwaave.co';

    const API_PROD_URL = 'https://getwaave.co';
    const API_SANDBOX_URL = 'https://staging.getwaave.co';

    protected $logger;

    protected $validation_code;

    public $order_button_text = "Place order";

    /**
     * Constructor for the gateway.
     */
    public function __construct() {
        $this->id                 = 'waave_checkout';
        $this->method_title       = 'WAAVE Complete';
        $this->method_description = 'WAAVE Checkout';
      
        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();

        // WAAVE values
        $this->url = self::PROD_URL;
        $this->api_url = self::API_PROD_URL;
        if ( 'yes' === $this->get_option( 'testmode' ) ) {
            $this->url = self::SANDBOX_URL;
            $this->api_url = self::API_SANDBOX_URL;
        }
        $this->access_key   = $this->get_option( 'access_key' );
        $this->venue_id     = $this->get_option( 'venue_id' );
        $this->callback_url = add_query_arg( 'wc-api', 'WC_WAAVE_Checkout', home_url( '/' ) );

        // Common fields
        $this->title        = 'WAAVE Checkout: Credit & Debit Cards, Fast & Safe';
        $this->description  = '<p>Payment protection program on all your transactions. US & International major credit and debit cards. More about <a href="https://www.getwaave.com/what-is-waave" target="blank">WAAVE</a></p>' .
            '<p style="margin-top: 10px">' . $this->get_option( 'description' ) . '</p>';

        // Actions
        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
        add_action( 'woocommerce_receipt_waave_checkout', array( $this, 'receipt_page' ) );
        add_action( 'woocommerce_api_wc_waave_checkout', array( $this, 'check_waave_response' ) );
        add_filter( 'woocommerce_get_checkout_payment_url', array($this, 'filter_payment_url') );
    }

    public function filter_payment_url($pay_url) {
        return add_query_arg( 'validation_code', $this->validation_code, $pay_url );
    }

    /**
     * Initialize Gateway Settings Form Fields
     */
    public function init_form_fields() {
        $this->form_fields = apply_filters( 'wc_waave_form_fields', array(
      
            'enabled' => array(
                'title'   => __( 'Enable/Disable', 'wc-gateway-waave' ),
                'type'    => 'checkbox',
                'label'   => __( 'Enable WAAVE Checkout', 'wc-gateway-waave' ),
                'default' => 'yes'
            ),

            'testmode' => array(
                'title'       => __( 'WAAVE Sandbox', 'wc-gateway-waave' ),
                'type'        => 'checkbox',
                'label'       => __( 'WAAVE Sandbox', 'wc-gateway-waave' ),
                'description' => __( 'Place the payment gateway in the development mode.' ),
                'default'     => 'yes'
            ),
            
            'access_key' => array(
                'title'       => __( 'Access Key', 'wc-gateway-waave' ),
                'type'        => 'text',
                'description' => __( 'This is an access key for WAAVE connection.', 'wc-gateway-waave' ),
                'desc_tip'    => true
            ),

            'private_key' => array(
                'title'       => __( 'Private Key', 'wc-gateway-waave' ),
                'type'        => 'text',
                'description' => __( 'This is an private key for WAAVE connection.', 'wc-gateway-waave' ),
                'desc_tip'    => true
            ),

            'venue_id' => array(
                'title'       => __( 'Venue ID', 'wc-gateway-waave' ),
                'type'        => 'text',
                'description' => __( 'This is a venue id for WAAVE connection.', 'wc-gateway-waave' ),
                'desc_tip'    => true
            ),

            'description' => array(
                'title'       => __( 'Description', 'wc-gateway-waave' ),
                'type'        => 'textarea',
                'description' => __( 'This will appear below WAAVE payemt button.', 'wc-gateway-waave' ),
                'desc_tip'    => true
            )
        ) );
    }

    /**
     * Get_icon function.
     *
     * @since 1.0.0
     * @version 4.0.0
     * @return string
     */
    public function get_icon() {
        $icons_str = '<img src="' . WC_WAAVE_PLUGIN_URL . '/assets/images/logo.png" class="" width="100" alt="WAAVE" />';

        return apply_filters( 'woocommerce_gateway_icon', $icons_str, $this->id );
    }

    /**
     * {@inheritdoc}
     */
    public function validate_fields() {
        $products = array();
        $cart_contents = WC()->cart->get_cart();
        foreach ($cart_contents as $value) {
            $product = $value['data'];
            if (empty($product)) {
                continue;
            }

            $product_id = $product->get_id();
            $categories = $product->get_category_ids();
            if ('variation' == $product->get_type()) {
                $product_id = $product->get_parent_id();
                $parent = wc_get_product($product_id);
                $categories = $parent->get_category_ids();
            }

            $products[] = array(
                'id' => $product_id,
                'name' => $product->get_name(),
                'sku' => $product->get_sku(),
                'price' => $product->get_price(),
                'quantity' => $value['quantity'],
                'categories' => $categories
            );
        }

        $shipping_first_name = $this->get_posted_value('billing_first_name');
        $shipping_last_name = $this->get_posted_value('billing_last_name');
        $shipping_country = $this->get_posted_value('billing_country');
        $shipping_city = $this->get_posted_value('billing_city');
        $shipping_state = $this->get_posted_value('billing_state', '');
        if (empty($shipping_state)) {
            $shipping_state = "MUnicipality";
        }
        $shipping_address_1 = $this->get_posted_value('billing_address_1');
        $shipping_address_2 = $this->get_posted_value('billing_address_2');
        $shipping_postcode = $this->get_posted_value('billing_postcode');

        $billing = array(
            'billing_first_name' => $shipping_first_name,
            'billing_last_name' => $shipping_last_name,
            'billing_country' => $shipping_country,
            'billing_country' => $shipping_country,
            'billing_address_1' => $shipping_address_1,
            'billing_address_2' => $shipping_address_2,
            'billing_city' => $shipping_city,
            'billing_state' => $shipping_state,
            'billing_postcode' => $shipping_postcode
        );

        $ship_to_different_address = $this->get_posted_value('ship_to_different_address', 0);
        if ($ship_to_different_address) {
            $shipping_country = $this->get_posted_value('shipping_country');
            $shipping_city = $this->get_posted_value('shipping_city');
            $shipping_state = $this->get_posted_value('shipping_state', '');
            if (empty($shipping_state)) {
                $shipping_state = "MUnicipality";
            }
            $shipping_address_1 = $this->get_posted_value('shipping_address_1');
            $shipping_address_2 = $this->get_posted_value('shipping_address_2');
            $shipping_postcode = $this->get_posted_value('shipping_postcode');
        }

        $shipping = array(
            'shipping_country' => $shipping_country,
            'shipping_address_1' => $shipping_address_1,
            'shipping_address_2' => $shipping_address_2,
            'shipping_city' => $shipping_city,
            'shipping_state' => $shipping_state,
            'shipping_postcode' => $shipping_postcode
        );

        $venue_id = get_option('waave_compliance_venue_id');
        if (is_plugin_inactive('waave-compliance/waave-compliance.php')) {
            $venue_id = $this->venue_id;
        }

        $body = array(
            'venue_id' => $venue_id,
            'amount'   => WC()->cart->get_cart_contents_total(),
            'email'    => $this->get_posted_value('billing_email'),
            'products' => $products,
            'shipping' => $shipping,
            'billing'  => $billing
        );

        $url = $this->api_url . '/compliance/validate';
        $options = array(
            'body' => $body
        );

        $request  = wp_remote_post( $url, $options );
        $response = json_decode( wp_remote_retrieve_body( $request ), true );

        if (isset($response['message']) && empty($response['success'])) {
            wc_add_notice( '<strong>' . $response['message'] . '</strong>', 'error' );
            return false;
        }

        // Save validation code
        if (isset($response['validation_code'])) {
            $this->validation_code = $response['validation_code'];
        }

        return true;
    }

    /**
     * Process the payment and return the result
     *
     * @param int $order_id
     * @return array
     */
    public function process_payment( $order_id ) {
        $order = wc_get_order( $order_id );

        // Return receipt page
        return array(
            'result'    => 'success',
            'redirect'  => $order->get_checkout_payment_url( true )
        );
    }


    /**
     * Reciept page.
     *
     * Display text and a button to direct the user to WAAVE.
     *
     * @since 1.0.0
     */
    public function receipt_page( $order ) {
        echo '<p>' . __( 'Thank you for your order, please click the button below to pay with WAAVE.', 'woocommerce-gateway-waave' ) . '</p>';
        echo $this->generate_waave_form( $order );
    }

    /**
     * Generate the WAAVE button link.
     *
     * @since 1.0.0
     */
    public function generate_waave_form( $order_id ) {
        $order = wc_get_order( $order_id );

        // Construct variables for post
        $data_to_send = array(
            // WAAVE details
            'access_key'   => $this->access_key,
            'return_url'   => $this->get_return_url( $order ),
            'cancel_url'   => wc_get_cart_url(),
            'callback_url' => $this->callback_url,

            // Order details
            'amount'       => $order->get_total(),
            'reference_id' => $order->get_id(),
            'currency'     => $order->get_currency(),

            // Inconsitent data
            'venue_id'     => $this->venue_id,
            'validation_code' => $_GET['validation_code'],
        );

        $waave_args_array = array();
        foreach ( $data_to_send as $key => $value ) {
            $waave_args_array[] = '<input type="hidden" name="' . esc_attr( $key ) . '" value="' . esc_attr( $value ) . '" />';
        }

        return '<form action="' . esc_url( $this->url . '/waavepay/checkout' ) . '" method="get" id="waave_payment_form">
                ' . implode( '', $waave_args_array ) . '
                <input type="submit" class="button-alt" id="submit_waave_payment_form" value="' . __( 'Pay via WAAVE', 'woocommerce-gateway-waave' ) . '" /> <a class="button cancel" href="' . $order->get_cancel_order_url() . '">' . __( 'Cancel order &amp; restore cart', 'woocommerce-gateway-waave' ) . '</a>
                <script type="text/javascript">
                    jQuery(function(){
                        jQuery("body").block(
                            {
                                message: "' . __( 'Thank you for your order. We are now redirecting you to WAAVE to make payment.', 'woocommerce-gateway-waave' ) . '",
                                overlayCSS:
                                {
                                    background: "#fff",
                                    opacity: 0.6
                                },
                                css: {
                                    padding:        20,
                                    textAlign:      "center",
                                    color:          "#555",
                                    border:         "3px solid #aaa",
                                    backgroundColor:"#fff",
                                    cursor:         "wait"
                                }
                            });
                        jQuery( "#submit_waave_payment_form" ).click();
                    });
                </script>
            </form>';
    }

    /**
     * Check WAAVE response.
     *
     * @since 1.0.0
     */
    public function check_waave_response() {
        $json = file_get_contents('php://input');
        $data = json_decode($json, true);

        $data = stripslashes_deep( $data );

        $this->log( 'WAAVE Data: ' . print_r( $data, true ) );

        ob_end_clean();

        $valid = $this->validate_signature( $data );
        if ( $valid ) {
            $this->validate_request_data( $data );
        }

        $this->log( 'Check status and update order' );

        $order_id = absint( $data['reference_id'] );
        $order    = wc_get_order( $order_id );

        $status = $data['status'];
        if ( 'completed' === $status ) {
            $this->handle_payment_completed( $order );
        } elseif ( 'pending' === $status ) {
            $this->handle_payment_pending( $order );
        } elseif ( 'cancelled' === $status ) {
            $this->handle_payment_cancelled( $order );
        }

        // Notify WAAVE that information has been received
        die('Successfully updated order status.');
    }

    /**
     * Validate signature
     * 
     * @param array $data
     */
    private function validate_signature( $data ) {
        $secretKey  = $this->get_option( 'private_key' );
        $uri        = $this->callback_url;
        $body       = json_encode($data);

        $signature        = hash( "sha256", $secretKey . $uri . $body );
        $header_signature = isset( $_SERVER['HTTP_X_API_SIGNATURE'] ) ? $_SERVER['HTTP_X_API_SIGNATURE'] : '';

        if ($signature === $header_signature) {
            $this->log( 'Signature is valid.' );
            return true;
        }

        $this->log( 'Signature is invalid.' );
        $this->log( 'Woo signature: ' . $signature );
        $this->log( 'Woo secret key: ' . $secretKey );
        $this->log( 'Woo uri: ' . $uri );
        $this->log( 'Woo body: ' . $body );

        die('Signature is invalid.');
    }


    /**
     * Validate request data
     *
     * @param array $data
     */
    private function validate_request_data( $data ) {
        $order_id = absint( $data['reference_id'] );
        $order    = wc_get_order( $order_id );

        if ( ! $order ) {
            $this->log( 'Order is not exist.' );
            die( 'Order is not exist.' );
        }

        $amount = $order->get_total();
        if ($amount != $data['amount']) {
            $this->log( 'Amount is invalid.' );
            die( 'Amount is invalid.' );
        }

        if ( 'completed' == $order->get_status() ) {
            $this->log( 'Order has already been processed.' );
            die( 'Order has already been processed.' );
        }

        $this->log( 'Response data is valid.' );
        return true;
    }

    
    /**
     * Handle payment completed
     */
    private function handle_payment_completed( $order ) {
        $this->log( '- Completed' );
        $order->add_order_note( __( 'WAAVE payment completed', 'woocommerce-gateway-waave' ) );
        $order->payment_complete();
    }

    /**
     * Handle payment pending
     */
    private function handle_payment_pending( $order ) {
        $this->log( '- Pending' );
        $order->update_status( 'on-hold', __( 'This payment is pending via WAAVE.', 'woocommerce-gateway-waave' ) );
    }

    /**
     * Handle payment cancelled
     */
    private function handle_payment_cancelled( $order ) {
        $this->log( '- Cancelled' );
        $order->update_status( 'cancelled', __( 'This payment has cancelled via WAAVE.', 'woocommerce-gateway-waave' ) );
    }

    /**
     * Log system processes.
     * @since 1.0.0
     */
    private function log( $message ) {
        if ( empty( $this->logger ) ) {
            $this->logger = new WC_Logger();
        }

        $this->logger->add( 'waave', $message );
    }

    /**
     * Get POST data
     */
    private function get_posted_value( $key, $default = '') {
        $value = $default;

        if ( isset( $_POST[ $key ] ) ) {
            $value = is_string( $_POST[ $key ] ) ? trim( $_POST[ $key ] ) : $_POST[ $key ];
        }

        return $value;
    }
}
